"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
exports.defaults = {
    fps: 60,
    spectrumWidth: '40%',
    spectrumHeight: '10%',
    spectrumX: 'center',
    spectrumY: 'top',
    spectrumRotation: 'down',
    spectrumBusesCount: 64,
    spectrumBusMargin: 4,
    spectrumOpacity: '100%',
};
var postitionAliases = [
    { name: 'left', value: 0 },
    { name: 'center', value: 0.5 },
    { name: 'right', value: 1 },
    { name: 'top', value: 0 },
    { name: 'middle', value: 0.5 },
    { name: 'bottom', value: 1 },
];
exports.rotationAliasValues = ['up', 'down'];
exports.checkIsPercentValue = function (value) {
    return RegExp(/\d+%/).test(value);
};
exports.getAudioFilePath = function (config) {
    return path_1.default.resolve(config.audio.path);
};
exports.getBackgroundImagePath = function (config) {
    return path_1.default.resolve(config.image.path);
};
exports.getOutVideoPath = function (config) {
    return path_1.default.resolve(config.outVideo.path);
};
exports.getFPS = function (config) {
    return config.outVideo.fps || exports.defaults.fps;
};
exports.getSpectrumBusesCount = function () { return exports.defaults.spectrumBusesCount; };
exports.getSpectrumBusMargin = function () { return exports.defaults.spectrumBusMargin; };
var getSpectrumWidth = function (config) {
    return (config.outVideo.spectrum && config.outVideo.spectrum.width) ||
        exports.defaults.spectrumWidth;
};
var getSpectrumSizeAbsolute = function (spectrumSize, backgroundImageSize, fieldName) {
    if (typeof spectrumSize === 'number') {
        return spectrumSize;
    }
    var isPercentValue = exports.checkIsPercentValue(spectrumSize);
    if (!isPercentValue) {
        throw new Error(getSpectrumSizeValueErrorMessage(fieldName, spectrumSize));
    }
    var percentValue = parseInt(spectrumSize) / 100;
    return backgroundImageSize * percentValue;
};
var getSpectrumSizeValueErrorMessage = function (fieldName, value) {
    return "Invalid spectrum " + fieldName + " value: '" + value + "'. Use number value or percent value in string, for example: '30%'.";
};
exports.getSpectrumWidthAbsolute = function (config, backgroundImageWidth) {
    var spectrumWidth = getSpectrumWidth(config);
    var spectrumWidthAbsolute = getSpectrumSizeAbsolute(spectrumWidth, backgroundImageWidth, 'width');
    var spectrumWidthWithoutMargin = Math.trunc(spectrumWidthAbsolute / exports.defaults.spectrumBusesCount) - exports.defaults.spectrumBusMargin / 2;
    if (spectrumWidthWithoutMargin <= 0) {
        throw new Error("Spectrum width '" + spectrumWidth + "' is too small.");
    }
    return spectrumWidthAbsolute;
};
var getSpectrumHeight = function (config) {
    return (config.outVideo.spectrum && config.outVideo.spectrum.height) ||
        exports.defaults.spectrumHeight;
};
exports.getSpectrumHeightAbsolute = function (config, backgroundImageHeight) {
    var spectrumHeight = getSpectrumHeight(config);
    return getSpectrumSizeAbsolute(spectrumHeight, backgroundImageHeight, 'height');
};
var getSpectrumX = function (config) {
    return parseNumberIfPossible((config.outVideo.spectrum && config.outVideo.spectrum.x) ||
        exports.defaults.spectrumX);
};
var getSpectrumY = function (config) {
    return parseNumberIfPossible((config.outVideo.spectrum && config.outVideo.spectrum.y) ||
        exports.defaults.spectrumY);
};
var parseNumberIfPossible = function (value) {
    var parsed = Number(value);
    if (isNaN(parsed)) {
        return value;
    }
    return parsed;
};
var parsePositionAlias = function (aliasName) {
    var nameNormalized = aliasName.trim().toLowerCase();
    var alias = postitionAliases.find(function (als) { return als.name === nameNormalized; });
    if (!alias) {
        throw new Error();
    }
    return alias;
};
var getValidPositionAliasValues = function () {
    return postitionAliases.map(function (aliasEl) { return aliasEl.name; }).join(', ');
};
var getSpectrumAbsolutePosition = function (relativePosition, spectrumSize, backgroundImageSize) {
    switch (relativePosition) {
        case 0:
            return 0;
        case 0.5:
            return relativePosition * backgroundImageSize - spectrumSize / 2;
        case 1:
            return relativePosition * backgroundImageSize - spectrumSize;
        default:
            throw new Error();
    }
};
exports.getSpectrumXAbsolute = function (config, spectrumWidth, backgroundImageWidth) {
    var spectrumX = getSpectrumX(config);
    if (typeof spectrumX !== 'string') {
        return spectrumX;
    }
    try {
        var spectrumXParsed = parsePositionAlias(spectrumX);
        return getSpectrumAbsolutePosition(spectrumXParsed.value, spectrumWidth, backgroundImageWidth);
    }
    catch (_a) {
        throw new Error("Invalid spectrum x value: " + spectrumX + ". Valid values: " + getValidPositionAliasValues() + ".");
    }
};
exports.getSpectrumYAbsolute = function (config, spectrumHeight, backgroundImageHeight) {
    var spectrumY = getSpectrumY(config);
    if (typeof spectrumY !== 'string') {
        return spectrumY;
    }
    try {
        var spectrumXParsed = parsePositionAlias(spectrumY);
        return getSpectrumAbsolutePosition(spectrumXParsed.value, spectrumHeight, backgroundImageHeight);
    }
    catch (_a) {
        throw new Error("Invalid spectrum y value: " + spectrumY + ". Valid values: " + getValidPositionAliasValues() + ".");
    }
};
var getValidRotationAliasValues = function () {
    return exports.rotationAliasValues.join(', ');
};
exports.checkIsValidRotationAlias = function (aliasName) {
    return !!exports.rotationAliasValues.find(function (al) { return al === aliasName; });
};
exports.getSpectrumRotation = function (config) {
    var rotationValue = (config.outVideo.spectrum && config.outVideo.spectrum.rotation) ||
        exports.defaults.spectrumRotation;
    if (!exports.checkIsValidRotationAlias(rotationValue)) {
        throw new Error("Invalid spectrum rotation value: " + rotationValue + ". Valid values: " + getValidRotationAliasValues + ".");
    }
    return rotationValue;
};
exports.getSpectrumColor = function (config) {
    return config.outVideo.spectrum && config.outVideo.spectrum.color;
};
exports.getSpectrumOpacity = function (config) {
    return (config.outVideo.spectrum && config.outVideo.spectrum.opacity) ||
        exports.defaults.spectrumOpacity;
};
exports.getSpectrumOpacityParsed = function (config) {
    var spectrumOpacity = exports.getSpectrumOpacity(config);
    if (!exports.checkIsPercentValue(spectrumOpacity)) {
        throw new Error("Invalid spectrum opacity value: '" + spectrumOpacity + "'. Use string percent value, for example '80%'.");
    }
    var percentValue = parseInt(spectrumOpacity);
    if ((percentValue < 0) ||
        (percentValue > 100)) {
        throw new Error("Invalid spectrum opacity value: '" + spectrumOpacity + "'. Percent values must be in range from '0%' to '100%'.");
    }
    return percentValue / 100;
};
exports.getFfmpeg_cfr = function (config) {
    return config.tweaks && config.tweaks.ffmpeg_cfr;
};
exports.getFfmpeg_preset = function (config) {
    return config.tweaks && config.tweaks.ffmpeg_preset;
};
exports.getFrame_processing_delay = function (config) {
    return config.tweaks && config.tweaks.frame_processing_delay;
};
//# sourceMappingURL=config.js.map